import re
import molgrid
import numpy as np
import torch
import torch.nn as nn
import torch.nn.functional as F
import torch.optim as optim
from torch.nn import init
from torch import autograd



from scipy.stats import pearsonr
import matplotlib.pyplot as plt
import matplotlib as mpl

path_train_lig_cache = '/net/pulsar/home/koes/liyeny/dl_project/molcache_file/lig_train0.molcache2'
path_train_rec_cache ='/net/pulsar/home/koes/liyeny/dl_project/molcache_file/rec_train0.molcache2'
path_train_type = '/net/pulsar/home/koes/liyeny/dl_project/type_files/gaffwmintrain0.types'

path_test_lig_cache = '/net/pulsar/home/koes/liyeny/dl_project/molcache_file/lig_test0.molcache2'
path_test_rec_cache = '/net/pulsar/home/koes/liyeny/dl_project/molcache_file/rec_test0.molcache2'
path_test_type = '/net/pulsar/home/koes/liyeny/dl_project/type_files/gaffwmintest0.types'

lr = 0.001
dropout = 0
momentum = 0.9
solver = "adam"
epochs = 200
batch_norm = 0
weight_decay = 0
clip = 0
binary_rep = False
extra_stats = False

hidden_size = 128
batch_size = 16
rotation_loss_weight = 1.0
absolute_loss_weight = 1.0
stratify_rec = False
iter_scheme = "small"


def weights_init(m):
    if isinstance(m, nn.Conv3d) or isinstance(m, nn.Linear):
        init.xavier_uniform_(m.weight.data)
        if m.bias is not None:
            init.constant_(m.bias.data, 0)

            
traine = molgrid.ExampleProvider(ligmolcache=path_train_lig_cache, recmolcache=path_train_rec_cache, shuffle=True, default_batch_size=batch_size, iteration_scheme = molgrid.IterationScheme.SmallEpoch, duplicate_first=False, balanced=True) ##a pair of ligand protein
traine.populate(path_train_type) ##provide the affinity

teste = molgrid.ExampleProvider(ligmolcache=path_test_lig_cache, recmolcache=path_test_rec_cache, shuffle=True, default_batch_size=batch_size, iteration_scheme=molgrid.IterationScheme.SmallEpoch, duplicate_first=False, balanced=True)
teste.populate(path_test_type)

gmaker = molgrid.GridMaker(binary=binary_rep) #binary ture-- point
dims = gmaker.grid_dimensions(14*2) # 14 atoms -- 1 protein 1 ligand # only one rec+onelig per example
tensor_shape = (batch_size,)+dims


class View(nn.Module):
        def __init__(self,shape):
                super(View, self).__init__()
                self.shape = shape
        def forward(self, input):
                return input.view(*self.shape)

class Net(nn.Module):
        def __init__(self, dims):
                super(Net, self).__init__()
                self.modules = []
                nchannels = dims[0]

                self.func = F.relu

                avgpool1 = nn.AvgPool3d(2,stride=2)
                self.add_module('avgpool_0', avgpool1)
                self.modules.append(avgpool1)
                conv1 = nn.Conv3d(nchannels,out_channels=32,padding=1,kernel_size=3,stride=1) 
                self.add_module('unit1_conv',conv1)
                self.modules.append(conv1)
                conv2 = nn.Conv3d(32,out_channels=32,padding=0,kernel_size=1,stride=1) 
                self.add_module('unit2_conv',conv2)
                self.modules.append(conv2)
                avgpool2 = nn.AvgPool3d(2,stride=2)
                self.add_module('avgpool_1', avgpool2)
                self.modules.append(avgpool2)
                conv3 = nn.Conv3d(32,out_channels=64,padding=1,kernel_size=3,stride=1) 
                self.add_module('unit3_conv',conv3)
                self.modules.append(conv3)
                conv4 = nn.Conv3d(64,out_channels=64,padding=0,kernel_size=1,stride=1) 
                self.add_module('unit4_conv',conv4)
                self.modules.append(conv4)
                avgpool3 = nn.AvgPool3d(2,stride=2)
                self.add_module('avgpool_2', avgpool3)
                self.modules.append(avgpool3)
                conv5 = nn.Conv3d(64,out_channels=128,padding=1,kernel_size=3,stride=1) 
                self.add_module('unit5_conv',conv5)
                self.modules.append(conv5)
                div = 2*2*2
                last_size = int(dims[1]//div * dims[2]//div * dims[3]//div * 128)
                print(last_size)
                flattener = View((-1,last_size))
                self.add_module('flatten',flattener)
                self.modules.append(flattener)
                self.fc = nn.Linear(last_size,1)
                self.add_module('last_fc',self.fc)

        def forward(self, x): #should approximate the affinity of the receptor/ligand pair
                for layer in self.modules:
                        x= layer(x)
                        if isinstance(layer,nn.Conv3d):
                                x=self.func(x)
                return self.fc(x)

            
model = Net(dims)


if torch.cuda.device_count() > 1:
        print("Using {} GPUs".format(torch.cuda.device_count()))
        model = nn.DataParallel(model)
else:
        print('GPUS: {}'.format(torch.cuda.device_count()), flush=True)
        
model.to('cuda:0')
model.apply(weights_init)





def new_loss(outputs, labels):
    
    outputs = outputs.flatten()
    labels = labels.flatten()
    
    loss = torch.tensor((0.0), device='cuda')
    
    for output, label in zip(outputs, labels):

        if label < 0:
            abs_label = - label
            if output> abs_label:
                loss += (output-abs_label)**2      
        else:
            
            loss += (output-label)**2  
            
    
           
    return loss/len(outputs)




optimizer = optim.SGD(model.parameters(), lr=lr, weight_decay=weight_decay)
if solver == "adam":
    optimizer = optim.Adam(model.parameters(), lr=lr, weight_decay=weight_decay)
    

scheduler = optim.lr_scheduler.StepLR(optimizer, step_size=80, gamma=0.1)

input_tensor = torch.zeros(tensor_shape, dtype=torch.float32, device='cuda')

affinity_labels = torch.zeros(batch_size, dtype=torch.float32, device ='cuda')



losses = []
for e in range(epochs):
    for batch in traine:
        optimizer.zero_grad()
        batch.extract_label(1, affinity_labels)
        labels = affinity_labels.float().to('cuda')
        gmaker.forward(batch, input_tensor, random_translation=2, random_rotation=True)
        output = model(input_tensor)
        loss = new_loss(output,labels)
        print("loss is:", loss)
        loss.backward()
        nn.utils.clip_grad_norm_(model.parameters(),1)
        optimizer.step()
        losses.append(float(loss))
    scheduler.step() 
    print("this is the:", e, "epoch")
    print("current learning rate:", scheduler.get_last_lr())

    
plt.plot(losses, color="blue")
plt.savefig('training_losses.png')


ypoints = np.array(losses)
plt.ylim([0, 6])
plt.plot(ypoints, color="blue")
plt.savefig('scaled_training_losses.png')








