import re
import molgrid
import numpy as np
import torch
import torch.nn as nn
import torch.nn.functional as F
import torch.optim as optim
from torch.nn import init
from torch import autograd



from scipy.stats import pearsonr
import matplotlib.pyplot as plt
import matplotlib as mpl

import seaborn as sns

path_train_lig_cache = '/net/pulsar/home/koes/liyeny/dl_project/molcache_file/lig_train0.molcache2'
path_train_rec_cache ='/net/pulsar/home/koes/liyeny/dl_project/molcache_file/rec_train0.molcache2'
path_train_type = '/net/pulsar/home/koes/liyeny/dl_project/type_files/gaffwmintrain0.types'

path_test_lig_cache = '/net/pulsar/home/koes/liyeny/dl_project/molcache_file/lig_test0.molcache2'
path_test_rec_cache = '/net/pulsar/home/koes/liyeny/dl_project/molcache_file/rec_test0.molcache2'
path_test_type = '/net/pulsar/home/koes/liyeny/dl_project/type_files/gaffwmintest0.types'


lr = 0.001
dropout = 0
momentum = 0.9
solver = "adam"
epochs = 1
batch_norm = 0
weight_decay = 0
clip = 0
binary_rep = False
extra_stats = False

hidden_size = 128
batch_size = 32 

rotation_loss_weight = 1.0
absolute_loss_weight = 1.0
stratify_rec = False
iter_scheme = "small"

def weights_init(m):
    if isinstance(m, nn.Conv3d) or isinstance(m, nn.Linear):
        init.xavier_uniform_(m.weight.data)
        if m.bias is not None:
            init.constant_(m.bias.data, 0)

traine = molgrid.ExampleProvider(ligmolcache=path_train_lig_cache, recmolcache=path_train_rec_cache, shuffle=True, default_batch_size=batch_size, iteration_scheme = molgrid.IterationScheme.LargeEpoch, duplicate_first=False, balanced=True) ##a pair of ligand protein
traine.populate(path_train_type) ##provide the affinity

teste = molgrid.ExampleProvider(ligmolcache=path_test_lig_cache, recmolcache=path_test_rec_cache, shuffle=True, default_batch_size=1, iteration_scheme=molgrid.IterationScheme.LargeEpoch, duplicate_first=False, balanced=True)
teste.populate(path_test_type)

gmaker = molgrid.GridMaker(binary=binary_rep) #binary ture-- point
dims = gmaker.grid_dimensions(14*2) # 14 atoms -- 1 protein 1 ligand # only one rec+onelig per example
tensor_shape = (batch_size,)+dims



class View(nn.Module):
        def __init__(self,shape):
                super(View, self).__init__()
                self.shape = shape
        def forward(self, input):
                return input.view(*self.shape)

class Net(nn.Module):
        def __init__(self, dims):
                super(Net, self).__init__()
                self.modules = []
                nchannels = dims[0]

                self.func = F.relu

                avgpool1 = nn.AvgPool3d(2,stride=2)
                self.add_module('avgpool_0', avgpool1)
                self.modules.append(avgpool1)
                conv1 = nn.Conv3d(nchannels,out_channels=32,padding=1,kernel_size=3,stride=1) 
                self.add_module('unit1_conv',conv1)
                self.modules.append(conv1)
                conv2 = nn.Conv3d(32,out_channels=32,padding=0,kernel_size=1,stride=1) 
                self.add_module('unit2_conv',conv2)
                self.modules.append(conv2)
                avgpool2 = nn.AvgPool3d(2,stride=2)
                self.add_module('avgpool_1', avgpool2)
                self.modules.append(avgpool2)
                conv3 = nn.Conv3d(32,out_channels=64,padding=1,kernel_size=3,stride=1) 
                self.add_module('unit3_conv',conv3)
                self.modules.append(conv3)
                conv4 = nn.Conv3d(64,out_channels=64,padding=0,kernel_size=1,stride=1) 
                self.add_module('unit4_conv',conv4)
                self.modules.append(conv4)
                avgpool3 = nn.AvgPool3d(2,stride=2)
                self.add_module('avgpool_2', avgpool3)
                self.modules.append(avgpool3)
                conv5 = nn.Conv3d(64,out_channels=128,padding=1,kernel_size=3,stride=1) 
                self.add_module('unit5_conv',conv5)
                self.modules.append(conv5)
                div = 2*2*2
                last_size = int(dims[1]//div * dims[2]//div * dims[3]//div * 128)
                print(last_size)
                flattener = View((-1,last_size))
                self.add_module('flatten',flattener)
                self.modules.append(flattener)
                self.fc = nn.Linear(last_size,1)
                self.add_module('last_fc',self.fc)

        def forward(self, x): #should approximate the affinity of the receptor/ligand pair
                for layer in self.modules:
                        x= layer(x)
                        if isinstance(layer,nn.Conv3d):
                                x=self.func(x)
                return self.fc(x)
    

model = Net(dims)


if torch.cuda.device_count() > 1:
        print("Using {} GPUs".format(torch.cuda.device_count()))
        model = nn.DataParallel(model)
else:
        print('GPUS: {}'.format(torch.cuda.device_count()), flush=True)
        
model.to('cuda:0')    #####
model.apply(weights_init)










print("start to define loss")
def new_loss(outputs, labels):
    
    outputs = outputs.flatten()
    labels = labels.flatten()
    
    loss = torch.tensor((0.0), device='cuda')
    count = 0
    
    for output, label in zip(outputs, labels):
        if label < 0:
            abs_label = - label
    
            if output> abs_label:   
                loss += (output-abs_label)**2
                count +=1
        else:
            loss += (output-label)**2  
            count +=1
    return loss/count














print('------1-------------------')

optimizer = optim.SGD(model.parameters(), lr=lr, weight_decay=weight_decay)
if solver == "adam":
    optimizer = optim.Adam(model.parameters(), lr=lr, weight_decay=weight_decay)
    
print('------2-------------------')
scheduler = optim.lr_scheduler.StepLR(optimizer, step_size=80, gamma=0.1)



print('------3-------------------')
input_tensor = torch.zeros(tensor_shape, dtype=torch.float32, device='cuda')

affinity_labels = torch.zeros(batch_size, dtype=torch.float32, device ='cuda')

print('------4-------------------')

singleinput = torch.zeros((1,)+dims, dtype=torch.float32, device='cuda')

singleAffinity_label = torch.zeros(1,dtype=torch.float32,device='cuda')

print("---------------------------------------------")



def train_single_fold(model):
    
    losses = [] #losses for training for each batch 
    mses = [] #mean square error of test for each epoch during training 
    ames = [] #absolute mean error of test for each epoch during training
    pearsons = [] #correlation of test set(all samples) for each epoch during training

    for e in range(epochs): 
        print("start training", e, "epoch")
        for idx, batch in enumerate(traine):
            #print("this is the", idx, "batch for training")
            optimizer.zero_grad()
            batch.extract_label(1, affinity_labels)
            labels = affinity_labels.float().to('cuda')
            gmaker.forward(batch, input_tensor, random_translation=2, random_rotation=True)
            output = model(input_tensor)
            loss = new_loss(output,labels)
            #print("loss is:", loss)
            loss.backward()
            nn.utils.clip_grad_norm_(model.parameters(),1)
            optimizer.step()
            losses.append(float(loss))

        scheduler.step()
        #eval performance on test set
        with torch.no_grad():
            results = []
            testlabels = []
            for idx, batch in enumerate(teste):
                #print("this is the", idx, "example for validating")
                batch.extract_label(1,singleAffinity_label)
                
                if singleAffinity_label < 0:
                    continue
                    
                gmaker.forward(batch,singleinput)
                output = model(singleinput) ###
                results.append(output.detach().cpu().numpy())
                testlabels.append(singleAffinity_label.detach().cpu().numpy()) ###

            results = np.array(results).flatten()
            testlabels = np.array(testlabels).flatten()
            valrmse = np.sqrt(np.mean((results - testlabels)**2))  ####
            if np.isinf(valrmse):
                valrmse = 1000
            valame = np.mean(np.abs(results-testlabels))
            R = pearsonr(results,testlabels)[0]
            print("Validation",e,valrmse,valame,R)
            mses.append(valrmse)
            ames.append(valame)
            pearsons.append(R)
           

    return model,testlabels,results,mses,ames,pearsons,losses






model,testlabels,results,mses,ames,pearsons,losses = train_single_fold(model)

#plot losses
plt.plot(losses)
plt.savefig('training_losses4.png');

#plot scaled losses
plt.plot(losses)
plt.plot(range(99,len(losses)),np.convolve(losses, np.ones(100)/100,mode='valid')) #moving average
plt.xlabel('Iteration')
plt.ylabel('Loss')
plt.ylim(0,3)
plt.savefig('scaled_training_losses4.png');

#
plt.plot(pearsons)
plt.savefig('pearsons4.png');

#plot joint plot
j = sns.jointplot(x=testlabels,y=results)
plt.suptitle('R = %.2f'%pearsons[-1])
plt.savefig('joint_plot4.png');




















