#import libraries
print("importing packages")

import re
import molgrid
import numpy as np
import torch
import torch.nn as nn
import torch.nn.functional as F
import torch.optim as optim
from torch.nn import init
from torch import autograd

import wandb

from scipy.stats import pearsonr
import matplotlib.pyplot as plt
import matplotlib as mpl

import seaborn as sns




#Define the path
print("Defining the path")

path_train_lig_cache = '/net/pulsar/home/koes/liyeny/dl_project/molcache_file/lig_train0.molcache2'
path_train_rec_cache ='/net/pulsar/home/koes/liyeny/dl_project/molcache_file/rec_train0.molcache2'
path_train_type = '/net/pulsar/home/koes/liyeny/dl_project/type_files/gaffwmintrain0.types'

path_test_lig_cache = '/net/pulsar/home/koes/liyeny/dl_project/molcache_file/lig_test0.molcache2'
path_test_rec_cache = '/net/pulsar/home/koes/liyeny/dl_project/molcache_file/rec_test0.molcache2'
path_test_type = '/net/pulsar/home/koes/liyeny/dl_project/type_files/gaffwmintest0.types'



#log in wandb and setting up parameters for the model
print("logging in wandb and setting up parameters for the model")

wandb.login()



config = dict(
    lr = 0.001,
    dropout = 0,
    momentum = 0.9,
    solver = "adam",
    epochs = 200,
    batch_norm = 0,
    weight_decay = 0,
    binary_rep = False,
    batch_size = 32 
)

with wandb.init(project="dl_affinity", config= config):
    config = wandb.config

    
    

    
#Organize data into examples   
print("Organizing data into examples ")    

traine = molgrid.ExampleProvider(ligmolcache=path_train_lig_cache, recmolcache=path_train_rec_cache, shuffle=True, default_batch_size=config.batch_size, iteration_scheme = molgrid.IterationScheme.LargeEpoch, duplicate_first=False, balanced=True) ##a pair of ligand protein
traine.populate(path_train_type) ##provide the affinity

teste = molgrid.ExampleProvider(ligmolcache=path_test_lig_cache, recmolcache=path_test_rec_cache, shuffle=True, default_batch_size=1, iteration_scheme=molgrid.IterationScheme.LargeEpoch, duplicate_first=False, balanced=True)
teste.populate(path_test_type)

gmaker = molgrid.GridMaker(binary=config.binary_rep) #binary ture-- point
dims = gmaker.grid_dimensions(14*2) # 14 atoms -- 1 protein 1 ligand # only one rec+onelig per example
tensor_shape = (config.batch_size,)+dims





#define, create, and initialize the model
print("defining, creating, and initializing the model ")  
    
    
class View(nn.Module):
        def __init__(self,shape):
                super(View, self).__init__()
                self.shape = shape
        def forward(self, input):
                return input.view(*self.shape)

class Net(nn.Module):
        def __init__(self, dims):
                super(Net, self).__init__()
                self.modules = []
                nchannels = dims[0]

                self.func = F.relu

                avgpool1 = nn.AvgPool3d(2,stride=2)
                self.add_module('avgpool_0', avgpool1)
                self.modules.append(avgpool1)
                conv1 = nn.Conv3d(nchannels,out_channels=32,padding=1,kernel_size=3,stride=1) 
                self.add_module('unit1_conv',conv1)
                self.modules.append(conv1)
                conv2 = nn.Conv3d(32,out_channels=32,padding=0,kernel_size=1,stride=1) 
                self.add_module('unit2_conv',conv2)
                self.modules.append(conv2)
                avgpool2 = nn.AvgPool3d(2,stride=2)
                self.add_module('avgpool_1', avgpool2)
                self.modules.append(avgpool2)
                conv3 = nn.Conv3d(32,out_channels=64,padding=1,kernel_size=3,stride=1) 
                self.add_module('unit3_conv',conv3)
                self.modules.append(conv3)
                conv4 = nn.Conv3d(64,out_channels=64,padding=0,kernel_size=1,stride=1) 
                self.add_module('unit4_conv',conv4)
                self.modules.append(conv4)
                avgpool3 = nn.AvgPool3d(2,stride=2)
                self.add_module('avgpool_2', avgpool3)
                self.modules.append(avgpool3)
                conv5 = nn.Conv3d(64,out_channels=128,padding=1,kernel_size=3,stride=1) 
                self.add_module('unit5_conv',conv5)
                self.modules.append(conv5)
                div = 2*2*2
                last_size = int(dims[1]//div * dims[2]//div * dims[3]//div * 128)
                print(last_size)
                flattener = View((-1,last_size))
                self.add_module('flatten',flattener)
                self.modules.append(flattener)
                self.fc = nn.Linear(last_size,1)
                self.add_module('last_fc',self.fc)

        def forward(self, x): #should approximate the affinity of the receptor/ligand pair
                for layer in self.modules:
                        x= layer(x)
                        if isinstance(layer,nn.Conv3d):
                                x=self.func(x)
                return self.fc(x)
    

def weights_init(m):
    if isinstance(m, nn.Conv3d) or isinstance(m, nn.Linear):
        init.xavier_uniform_(m.weight.data)
        if m.bias is not None:
            init.constant_(m.bias.data, 0)

model = Net(dims)

if torch.cuda.device_count() > 1:
        print("Using {} GPUs".format(torch.cuda.device_count()))
        model = nn.DataParallel(model)
else:
        print('GPUS: {}'.format(torch.cuda.device_count()), flush=True)
        
model.to('cuda:0')
model.apply(weights_init)




#define custom loss
print("defining custom loss")

def new_loss(outputs, labels):
    
    outputs = outputs.flatten()
    labels = labels.flatten()
    
    loss = torch.tensor((0.0), device='cuda')
    count = 0
    
    for output, label in zip(outputs, labels):
        if label < 0:
            abs_label = - label
    
            if output> abs_label:   
                loss += (output-abs_label)**2
                count +=1
        else:
            loss += (output-label)**2  
            count +=1
    return loss/count



#define optimizer and scheduler
print("defining optimizer and scheduler")

optimizer = optim.SGD(model.parameters(), lr=config.lr, weight_decay=config.weight_decay)
if config.solver == "adam":
    optimizer = optim.Adam(model.parameters(), lr=config.lr, weight_decay=config.weight_decay)
    

scheduler = optim.lr_scheduler.StepLR(optimizer, step_size=80, gamma=0.1)




#define zero tensors to put things in
print("defining zero tensors to put things in")

input_tensor = torch.zeros(tensor_shape, dtype=torch.float32, device='cuda')
affinity_labels = torch.zeros(config.batch_size, dtype=torch.float32, device ='cuda')
singleinput = torch.zeros((1,)+dims, dtype=torch.float32, device='cuda')
singleAffinity_label = torch.zeros(1,dtype=torch.float32,device='cuda')




#define training loops and start training
print("defining training loops and start training")


def train_single_fold(model):
    wandb.init(project="dl_affinity", config= config)
    wandb.run.name = "fold0"
    
    losses = [] #losses for training for each batch 
    mses = [] #mean square error of test for each epoch during training 
    ames = [] #absolute mean error of test for each epoch during training
    pearsons = [] #correlation of test set(all samples) for each epoch during training

    for e in range(config.epochs): 
        print("start training", e, "epoch")
        for idx, batch in enumerate(traine):
            #print("this is the", idx, "batch for training")
            optimizer.zero_grad()
            batch.extract_label(1, affinity_labels)
            labels = affinity_labels.float().to('cuda')
            gmaker.forward(batch, input_tensor, random_translation=2, random_rotation=True)
            output = model(input_tensor)
            loss = new_loss(output,labels)
            #print("loss is:", loss)
            loss.backward()
            nn.utils.clip_grad_norm_(model.parameters(),1)
            optimizer.step()
            losses.append(float(loss))
            wandb.log({"train/loss":loss})

        scheduler.step()
        #eval performance on test set
        with torch.no_grad():
            results = []
            testlabels = []
            for idx, batch in enumerate(teste):
                #print("this is the", idx, "example for validating")
                batch.extract_label(1,singleAffinity_label)
                
                if singleAffinity_label < 0:
                    continue
                    
                gmaker.forward(batch,singleinput)
                output = model(singleinput) 
                results.append(output.detach().cpu().numpy())
                testlabels.append(singleAffinity_label.detach().cpu().numpy()) 

            results = np.array(results).flatten()
            testlabels = np.array(testlabels).flatten()
            
            #print("epochs:", e)
            #print("results:", results)
            #print("testlabels:", testlabels)
            
            valrmse = np.sqrt(np.mean((results - testlabels)**2))  
            if np.isinf(valrmse):
                valrmse = 1000
            valame = np.mean(np.abs(results-testlabels))

            
            
            
            R = pearsonr(results,testlabels)[0]
            #print("R:",R)
            print("Validation--","epoch:",e,"rmse:",valrmse,"ame:",valame,"R:",R)
            mses.append(valrmse)
            ames.append(valame)
            pearsons.append(R)
            wandb.log({"val/mses":valrmse, "epoch": e})
            wandb.log({"val/ames":valame, "epoch": e})
            wandb.log({"val/pearsons":R, "epoch": e})
        
           

    return model,testlabels,results,mses,ames,pearsons,losses



model,testlabels,results,mses,ames,pearsons,losses = train_single_fold(model)







#plotting and saving figures
print("plotting and saving figures")


plt.figure(0)
plt.plot(losses)
plt.savefig('/net/pulsar/home/koes/liyeny/dl_project/three_fold_results/fold0/training_losses.png')



plt.figure(1)
plt.plot(losses)
plt.plot(range(99,len(losses)),np.convolve(losses, np.ones(100)/100,mode='valid')) #moving average
plt.xlabel('Iteration')
plt.ylabel('Loss')
plt.ylim(0,3)
plt.savefig('/net/pulsar/home/koes/liyeny/dl_project/three_fold_results/fold0/scaled_training_losses.png')

plt.figure(2)
plt.plot(pearsons)
plt.savefig('/net/pulsar/home/koes/liyeny/dl_project/three_fold_results/fold0/pearsons.png')



plt.figure(3)
j = sns.jointplot(x=testlabels,y=results)
plt.suptitle('R = %.2f'%pearsons[-1])
plt.savefig('/net/pulsar/home/koes/liyeny/dl_project/three_fold_results/fold0/distribution.png')




#save lists
print("saving lists")

with open("/net/pulsar/home/koes/liyeny/dl_project/three_fold_results/fold0/losses.txt", "wb") as fp: 
    pickle.dump(losses, fp)

    
with open("/net/pulsar/home/koes/liyeny/dl_project/three_fold_results/fold0/pearsons.txt", "wb") as fp: 
    pickle.dump(pearsons, fp)
    
    
    
with open("/net/pulsar/home/koes/liyeny/dl_project/three_fold_results/fold0/ames.txt", "wb") as fp: 
    pickle.dump(ames, fp)
    
with open("/net/pulsar/home/koes/liyeny/dl_project/three_fold_results/fold0/mses.txt", "wb") as fp: 
    pickle.dump(mses, fp)



#end wandb

wandb.finish()

