import torch.nn as nn
import torch.nn.functional as F
import torch
import se3cnn
from se3cnn.image.convolution import SE3Convolution


class View(nn.Module):
        def __init__(self,shape):
                super(View, self).__init__()
                self.shape = shape
        def forward(self, input):
                return input.view(*self.shape)




class Net(nn.Module):
        def __init__(self, dims):
                super(Net, self).__init__()
                self.modules = []
                nchannels = dims[0]

                self.func = F.relu

                avgpool1 = nn.AvgPool3d(2,stride=2)
                self.add_module('avgpool_0', avgpool1)
                self.modules.append(avgpool1)
                
                conv1 = SE3Convolution([(28, 0)] , [(32, 0)], size=3, stride = 1, padding =1)
                self.add_module('unit1_conv',conv1)
                self.modules.append(conv1)
                
                conv2 = SE3Convolution([(32, 0)], [(32, 0)], size=1, stride = 1, padding =0)
                self.add_module('unit2_conv',conv2)
                self.modules.append(conv2)
                
                avgpool2 = nn.AvgPool3d(2,stride=2)
                self.add_module('avgpool_1', avgpool2)
                self.modules.append(avgpool2)
                
                conv3 = SE3Convolution([(32, 0)], [(64, 0)], size=3, stride = 1, padding =1)
                self.add_module('unit3_conv',conv3)
                self.modules.append(conv3)
                
                conv4 = SE3Convolution([(64, 0)], [(64, 0)] , size=1, stride = 1, padding =0) 
                self.add_module('unit4_conv',conv4)
                self.modules.append(conv4)
                
                avgpool3 = nn.AvgPool3d(2,stride=2)
                self.add_module('avgpool_2', avgpool3)
                self.modules.append(avgpool3)
                
                conv5 = SE3Convolution([(64, 0)], [(128, 0)], size=3, stride = 1, padding =1)
                self.add_module('unit5_conv',conv5)
                self.modules.append(conv5)
                
                div = 2*2*2
                last_size = int(dims[1]//div * dims[2]//div * dims[3]//div * 128)
                print(last_size)
                
                flattener = View((-1,last_size))
                self.add_module('flatten',flattener)
                self.modules.append(flattener)
                
                self.fc = nn.Linear(last_size,1)
                self.add_module('last_fc',self.fc)

        def forward(self, x): #should approximate the affinity of the receptor/ligand pair
                for layer in self.modules:
                        x= layer(x)
                        if isinstance(layer,se3cnn.image.convolution.SE3Convolution):
                                x=self.func(x)
                return self.fc(x)