import torch.nn as nn
import torch.nn.functional as F
import torch
import se3cnn
from se3cnn.image.convolution import SE3Convolution


class View(nn.Module):
        def __init__(self,shape):
                super(View, self).__init__()
                self.shape = shape
        def forward(self, input):
                return input.view(*self.shape)


#################### tensor model-- type A ############################################################################


class Net_A(nn.Module):
        def __init__(self, dims, ch_size):
                super(Net_A, self).__init__()
                self.modules = []
                #nchannels = dims[0]
                fac = int(ch_size/16)
                

                self.func = F.relu

                avgpool1 = nn.AvgPool3d(2,stride=2)
                self.add_module('avgpool_0', avgpool1)
                self.modules.append(avgpool1)
                
                conv1 = SE3Convolution([(28, 0)], [(3*fac, 0), (1*fac, 1), (2*fac , 2)], size=3,stride =1, padding =1)
                self.add_module('unit1_conv',conv1)
                self.modules.append(conv1)
                
                conv2 = SE3Convolution([(3*fac,0), (1*fac,1), (2*fac,2)], [(3*fac,0), (1*fac,1), (2*fac,2)], size=1, stride =1, padding =0)
                self.add_module('unit2_conv',conv2)
                self.modules.append(conv2)
                
                avgpool2 = nn.AvgPool3d(2,stride=2)
                self.add_module('avgpool_1', avgpool2)
                self.modules.append(avgpool2)
                
                conv3 = SE3Convolution([(3*fac,0), (1*fac,1), (2*fac,2)], [(6*fac,0), (2*fac,1), (4*fac,2)], size=3, stride =1, padding =1)
                self.add_module('unit3_conv',conv3)
                self.modules.append(conv3)
                
                conv4 = SE3Convolution([(6*fac,0), (2*fac,1), (4*fac,2)], [(6*fac,0), (2*fac,1), (4*fac,2)], size=1  ,stride =1, padding =0)
                self.add_module('unit4_conv',conv4)
                self.modules.append(conv4)
                
                avgpool3 = nn.AvgPool3d(2,stride=2)
                self.add_module('avgpool_2', avgpool3)
                self.modules.append(avgpool3)
                
                conv5 = SE3Convolution([(6*fac,0), (2*fac,1), (4*fac,2)], [(12*fac,0), (4*fac,1), (8*fac,2)], size=3,stride =1, padding =1)
                self.add_module('unit5_conv',conv5)
                self.modules.append(conv5)
                
                div = 2*2*2
                last_size = int(dims[1]//div * dims[2]//div * dims[3]//div * (12*fac*1 + 4*fac*3 + 8*fac*5)      ) 
                
                flattener = View((-1,last_size))
                self.add_module('flatten',flattener)
                self.modules.append(flattener)
                
                self.fc_aff = nn.Linear(last_size,1)
                self.fc_pose = nn.Linear(last_size,1)
                self.add_module('last_fc',self.fc_aff)
                
        def forward(self, x): #should approximate the affinity of the receptor/ligand pair
                for layer in self.modules:
                        x= layer(x)
                        if isinstance(layer,se3cnn.image.convolution.SE3Convolution):
                                x=self.func(x)
                affinity = self.fc_aff(x)  #affinity values
                pose = self.fc_pose(x) #logit predictions for the pose
                
                return affinity, pose
            
            
#################### tensor model-- type B ############################################################################            
            
            
class Net_B(nn.Module):
        def __init__(self, dims, ch_size):
                super(Net_B, self).__init__()
                self.modules = []
                #nchannels = dims[0]
                fac = int(ch_size/16)
                

                self.func = F.relu

                avgpool1 = nn.AvgPool3d(2,stride=2)
                self.add_module('avgpool_0', avgpool1)
                self.modules.append(avgpool1)
                
                conv1 = SE3Convolution([(28, 0)], [(2*fac, 0), (3*fac, 1), (1*fac , 2)], size=3,stride =1, padding =1)
                self.add_module('unit1_conv',conv1)
                self.modules.append(conv1)
                
                conv2 = SE3Convolution([(2*fac,0), (3*fac,1), (1*fac,2)], [(2*fac,0), (3*fac,1), (1*fac,2)], size=1, stride =1, padding =0)
                self.add_module('unit2_conv',conv2)
                self.modules.append(conv2)
                
                avgpool2 = nn.AvgPool3d(2,stride=2)
                self.add_module('avgpool_1', avgpool2)
                self.modules.append(avgpool2)
                
                conv3 = SE3Convolution([(2*fac,0), (3*fac,1), (1*fac,2)], [(4*fac,0), (6*fac,1), (2*fac,2)], size=3, stride =1, padding =1)
                self.add_module('unit3_conv',conv3)
                self.modules.append(conv3)
                
                conv4 = SE3Convolution([(4*fac,0), (6*fac,1), (2*fac,2)], [(4*fac,0), (6*fac,1), (2*fac,2)], size=1  ,stride =1, padding =0)
                self.add_module('unit4_conv',conv4)
                self.modules.append(conv4)
                
                avgpool3 = nn.AvgPool3d(2,stride=2)
                self.add_module('avgpool_2', avgpool3)
                self.modules.append(avgpool3)
                
                conv5 = SE3Convolution([(4*fac,0), (6*fac,1), (2*fac,2)], [(8*fac,0), (12*fac,1), (4*fac,2)], size=3,stride =1, padding =1)
                self.add_module('unit5_conv',conv5)
                self.modules.append(conv5)
                
                div = 2*2*2
                last_size = int(dims[1]//div * dims[2]//div * dims[3]//div * (8*fac*1 + 12*fac*3 + 4*fac*5)      ) 
                
                flattener = View((-1,last_size))
                self.add_module('flatten',flattener)
                self.modules.append(flattener)
                
                self.fc_aff = nn.Linear(last_size,1)
                self.fc_pose = nn.Linear(last_size,1)
                self.add_module('last_fc',self.fc_aff)

        def forward(self, x): #should approximate the affinity of the receptor/ligand pair
                for layer in self.modules:
                        x= layer(x)
                        if isinstance(layer,se3cnn.image.convolution.SE3Convolution):
                                x=self.func(x)
                affinity = self.fc_aff(x)  #affinity values
                pose = self.fc_pose(x) #logit predictions for the pose
                
                return affinity, pose
            
            

#################### tensor model-- type C ############################################################################            
            
            
class Net_C(nn.Module):
        def __init__(self, dims, ch_size):
                super(Net_C, self).__init__()
                self.modules = []
                #nchannels = dims[0]
                fac = int(ch_size/16)
                

                self.func = F.relu

                avgpool1 = nn.AvgPool3d(2,stride=2)
                self.add_module('avgpool_0', avgpool1)
                self.modules.append(avgpool1)
                
                conv1 = SE3Convolution([(28, 0)], [(5*fac, 0), (2*fac, 1), (1*fac , 2)], size=3,stride =1, padding =1)
                self.add_module('unit1_conv',conv1)
                self.modules.append(conv1)
                
                conv2 = SE3Convolution([(5*fac, 0), (2*fac, 1), (1*fac , 2)], [(5*fac, 0), (2*fac, 1), (1*fac,2)], size=1, stride =1, padding =0)
                self.add_module('unit2_conv',conv2)
                self.modules.append(conv2)
                
                avgpool2 = nn.AvgPool3d(2,stride=2)
                self.add_module('avgpool_1', avgpool2)
                self.modules.append(avgpool2)
                
                conv3 = SE3Convolution([(5*fac,0), (2*fac,1), (1*fac,2)], [(10*fac,0), (4*fac,1), (2*fac,2)], size=3, stride =1, padding =1)
                self.add_module('unit3_conv',conv3)
                self.modules.append(conv3)
                
                conv4 = SE3Convolution([(10*fac,0), (4*fac,1), (2*fac,2)], [(10*fac,0), (4*fac,1), (2*fac,2)], size=1  ,stride =1, padding =0)
                self.add_module('unit4_conv',conv4)
                self.modules.append(conv4)
                
                avgpool3 = nn.AvgPool3d(2,stride=2)
                self.add_module('avgpool_2', avgpool3)
                self.modules.append(avgpool3)
                
                conv5 = SE3Convolution([(10*fac,0), (4*fac,1),(2*fac,2)], [(20*fac,0), (8*fac,1), (4*fac,2)], size=3,stride =1, padding =1)
                self.add_module('unit5_conv',conv5)
                self.modules.append(conv5)
                
                div = 2*2*2
                last_size = int(dims[1]//div * dims[2]//div * dims[3]//div * (20*fac*1 + 8*fac*3 + 4*fac*5)      ) 
                
                flattener = View((-1,last_size))
                self.add_module('flatten',flattener)
                self.modules.append(flattener)
                
                self.fc_aff = nn.Linear(last_size,1)
                self.fc_pose = nn.Linear(last_size,1)
                self.add_module('last_fc',self.fc_aff)
                
        def forward(self, x): #should approximate the affinity of the receptor/ligand pair
                for layer in self.modules:
                        x= layer(x)
                        if isinstance(layer,se3cnn.image.convolution.SE3Convolution):
                                x=self.func(x)
                affinity = self.fc_aff(x)  #affinity values
                pose = self.fc_pose(x) #logit predictions for the pose
                
                return affinity, pose
            
            
#################### tensor model-- type D ############################################################################

class Net_D(nn.Module):
        def __init__(self, dims, ch_size):
                super(Net_D, self).__init__()
                self.modules = []
                #nchannels = dims[0]
                fac = int(ch_size/16)
                

                self.func = F.relu

                avgpool1 = nn.AvgPool3d(2,stride=2)
                self.add_module('avgpool_0', avgpool1)
                self.modules.append(avgpool1)
                
                conv1 = SE3Convolution([(28, 0)], [(0*fac, 0), (2*fac, 1), (2*fac , 2)], size=3,stride =1, padding =1)
                self.add_module('unit1_conv',conv1)
                self.modules.append(conv1)
                
                conv2 = SE3Convolution([(0*fac, 0), (2*fac, 1), (2*fac , 2)], [(0*fac, 0), (2*fac, 1), (2*fac,2)], size=1, stride =1, padding =0)
                self.add_module('unit2_conv',conv2)
                self.modules.append(conv2)
                
                avgpool2 = nn.AvgPool3d(2,stride=2)
                self.add_module('avgpool_1', avgpool2)
                self.modules.append(avgpool2)
                
                conv3 = SE3Convolution([(0*fac,0), (2*fac,1), (2*fac,2)], [(0*fac,0), (4*fac,1), (4*fac,2)], size=3, stride =1, padding =1)
                self.add_module('unit3_conv',conv3)
                self.modules.append(conv3)
                
                conv4 = SE3Convolution([(0*fac,0), (4*fac,1), (4*fac,2)], [(0*fac,0), (4*fac,1), (4*fac,2)], size=1  ,stride =1, padding =0)
                self.add_module('unit4_conv',conv4)
                self.modules.append(conv4)
                
                avgpool3 = nn.AvgPool3d(2,stride=2)
                self.add_module('avgpool_2', avgpool3)
                self.modules.append(avgpool3)
                
                conv5 = SE3Convolution([(0*fac,0), (4*fac,1),(4*fac,2)], [(0*fac,0), (8*fac,1), (8*fac,2)], size=3,stride =1, padding =1)
                self.add_module('unit5_conv',conv5)
                self.modules.append(conv5)
                
                div = 2*2*2
                last_size = int(dims[1]//div * dims[2]//div * dims[3]//div * (0*fac*1 + 8*fac*3 + 8*fac*5)      ) 
                
                flattener = View((-1,last_size))
                self.add_module('flatten',flattener)
                self.modules.append(flattener)
                
                self.fc_aff = nn.Linear(last_size,1)
                self.fc_pose = nn.Linear(last_size,1)
                self.add_module('last_fc',self.fc_aff)
                
        def forward(self, x): #should approximate the affinity of the receptor/ligand pair
                for layer in self.modules:
                        x= layer(x)
                        if isinstance(layer,se3cnn.image.convolution.SE3Convolution):
                                x=self.func(x)
                affinity = self.fc_aff(x)  #affinity values
                pose = self.fc_pose(x) #logit predictions for the pose
                
                return affinity, pose

#################### tensor model-- type E ############################################################################ 
class Net_E(nn.Module):
        def __init__(self, dims, ch_size):
                super(Net_E, self).__init__()
                self.modules = []
                #nchannels = dims[0]
                fac = int(ch_size/16)
                

                self.func = F.relu

                avgpool1 = nn.AvgPool3d(2,stride=2)
                self.add_module('avgpool_0', avgpool1)
                self.modules.append(avgpool1)
                
                conv1 = SE3Convolution([(28, 0)], [(6*fac, 0), (0*fac, 1), (2*fac , 2)], size=3,stride =1, padding =1)
                self.add_module('unit1_conv',conv1)
                self.modules.append(conv1)
                
                conv2 = SE3Convolution([(6*fac, 0), (0*fac, 1), (2*fac , 2)], [(6*fac, 0), (0*fac, 1), (2*fac,2)], size=1, stride =1, padding =0)
                self.add_module('unit2_conv',conv2)
                self.modules.append(conv2)
                
                avgpool2 = nn.AvgPool3d(2,stride=2)
                self.add_module('avgpool_1', avgpool2)
                self.modules.append(avgpool2)
                
                conv3 = SE3Convolution([(6*fac,0), (0*fac,1), (2*fac,2)], [(12*fac,0), (0*fac,1), (4*fac,2)], size=3, stride =1, padding =1)
                self.add_module('unit3_conv',conv3)
                self.modules.append(conv3)
                
                conv4 = SE3Convolution([(12*fac,0), (0*fac,1), (4*fac,2)], [(12*fac,0), (0*fac,1), (4*fac,2)], size=1  ,stride =1, padding =0)
                self.add_module('unit4_conv',conv4)
                self.modules.append(conv4)
                
                avgpool3 = nn.AvgPool3d(2,stride=2)
                self.add_module('avgpool_2', avgpool3)
                self.modules.append(avgpool3)
                
                conv5 = SE3Convolution([(12*fac,0), (0*fac,1),(4*fac,2)], [(24*fac,0), (0*fac,1), (8*fac,2)], size=3,stride =1, padding =1)
                self.add_module('unit5_conv',conv5)
                self.modules.append(conv5)
                
                div = 2*2*2
                last_size = int(dims[1]//div * dims[2]//div * dims[3]//div * (24*fac*1 + 0*fac*3 + 8*fac*5)      ) 
                
                flattener = View((-1,last_size))
                self.add_module('flatten',flattener)
                self.modules.append(flattener)
                
                self.fc_aff = nn.Linear(last_size,1)
                self.fc_pose = nn.Linear(last_size,1)
                self.add_module('last_fc',self.fc_aff)
                
        def forward(self, x): #should approximate the affinity of the receptor/ligand pair
                for layer in self.modules:
                        x= layer(x)
                        if isinstance(layer,se3cnn.image.convolution.SE3Convolution):
                                x=self.func(x)
                affinity = self.fc_aff(x)  #affinity values
                pose = self.fc_pose(x) #logit predictions for the pose
                
                return affinity, pose

#################### tensor model-- type F ############################################################################ 
class Net_F(nn.Module):
        def __init__(self, dims, ch_size):
                super(Net_F, self).__init__()
                self.modules = []
                #nchannels = dims[0]
                fac = int(ch_size/16)
                

                self.func = F.relu

                avgpool1 = nn.AvgPool3d(2,stride=2)
                self.add_module('avgpool_0', avgpool1)
                self.modules.append(avgpool1)
                
                conv1 = SE3Convolution([(28, 0)], [(8*fac, 0), (1*fac, 1), (1*fac , 2)], size=3,stride =1, padding =1)
                self.add_module('unit1_conv',conv1)
                self.modules.append(conv1)
                
                conv2 = SE3Convolution([(8*fac, 0), (1*fac, 1), (1*fac , 2)], [(8*fac, 0), (1*fac, 1), (1*fac,2)], size=1, stride =1, padding =0)
                self.add_module('unit2_conv',conv2)
                self.modules.append(conv2)
                
                avgpool2 = nn.AvgPool3d(2,stride=2)
                self.add_module('avgpool_1', avgpool2)
                self.modules.append(avgpool2)
                
                conv3 = SE3Convolution([(8*fac,0), (1*fac,1), (1*fac,2)], [(16*fac,0), (2*fac,1), (2*fac,2)], size=3, stride =1, padding =1)
                self.add_module('unit3_conv',conv3)
                self.modules.append(conv3)
                
                conv4 = SE3Convolution([(16*fac,0), (2*fac,1), (2*fac,2)], [(16*fac,0), (2*fac,1), (2*fac,2)], size=1  ,stride =1, padding =0)
                self.add_module('unit4_conv',conv4)
                self.modules.append(conv4)
                
                avgpool3 = nn.AvgPool3d(2,stride=2)
                self.add_module('avgpool_2', avgpool3)
                self.modules.append(avgpool3)
                
                conv5 = SE3Convolution([(16*fac,0), (2*fac,1),(2*fac,2)], [(32*fac,0), (4*fac,1), (4*fac,2)], size=3,stride =1, padding =1)
                self.add_module('unit5_conv',conv5)
                self.modules.append(conv5)
                
                div = 2*2*2
                last_size = int(dims[1]//div * dims[2]//div * dims[3]//div * (32*fac*1 + 4*fac*3 + 4*fac*5)      ) 
                
                flattener = View((-1,last_size))
                self.add_module('flatten',flattener)
                self.modules.append(flattener)
                
                self.fc_aff = nn.Linear(last_size,1)
                self.fc_pose = nn.Linear(last_size,1)
                self.add_module('last_fc',self.fc_aff)
                
        def forward(self, x): #should approximate the affinity of the receptor/ligand pair
                for layer in self.modules:
                        x= layer(x)
                        if isinstance(layer,se3cnn.image.convolution.SE3Convolution):
                                x=self.func(x)
                affinity = self.fc_aff(x)  #affinity values
                pose = self.fc_pose(x) #logit predictions for the pose
                
                return affinity, pose

#################### tensor model-- type G ############################################################################ 

class Net_G(nn.Module):
        def __init__(self, dims, ch_size):
                super(Net_G, self).__init__()
                self.modules = []
                #nchannels = dims[0]
                fac = int(ch_size/16)
                

                self.func = F.relu

                avgpool1 = nn.AvgPool3d(2,stride=2)
                self.add_module('avgpool_0', avgpool1)
                self.modules.append(avgpool1)
                
                conv1 = SE3Convolution([(28, 0)], [(11*fac, 0), (0*fac, 1), (1*fac , 2)], size=3,stride =1, padding =1)
                self.add_module('unit1_conv',conv1)
                self.modules.append(conv1)
                
                conv2 = SE3Convolution([(11*fac, 0), (0*fac, 1), (1*fac , 2)], [(11*fac, 0), (0*fac, 1), (1*fac,2)], size=1, stride =1, padding =0)
                self.add_module('unit2_conv',conv2)
                self.modules.append(conv2)
                
                avgpool2 = nn.AvgPool3d(2,stride=2)
                self.add_module('avgpool_1', avgpool2)
                self.modules.append(avgpool2)
                
                conv3 = SE3Convolution([(11*fac,0), (0*fac,1), (1*fac,2)], [(22*fac,0), (0*fac,1), (2*fac,2)], size=3, stride =1, padding =1)
                self.add_module('unit3_conv',conv3)
                self.modules.append(conv3)
                
                conv4 = SE3Convolution([(22*fac,0), (0*fac,1), (2*fac,2)], [(22*fac,0), (0*fac,1), (2*fac,2)], size=1  ,stride =1, padding =0)
                self.add_module('unit4_conv',conv4)
                self.modules.append(conv4)
                
                avgpool3 = nn.AvgPool3d(2,stride=2)
                self.add_module('avgpool_2', avgpool3)
                self.modules.append(avgpool3)
                
                conv5 = SE3Convolution([(22*fac,0), (0*fac,1),(2*fac,2)], [(44*fac,0), (0*fac,1), (4*fac,2)], size=3,stride =1, padding =1)
                self.add_module('unit5_conv',conv5)
                self.modules.append(conv5)
                
                div = 2*2*2
                last_size = int(dims[1]//div * dims[2]//div * dims[3]//div * (44*fac*1 + 0*fac*3 + 4*fac*5)      ) 
                
                flattener = View((-1,last_size))
                self.add_module('flatten',flattener)
                self.modules.append(flattener)
                
                self.fc_aff = nn.Linear(last_size,1)
                self.fc_pose = nn.Linear(last_size,1)
                self.add_module('last_fc',self.fc_aff)
                
        def forward(self, x): #should approximate the affinity of the receptor/ligand pair
                for layer in self.modules:
                        x= layer(x)
                        if isinstance(layer,se3cnn.image.convolution.SE3Convolution):
                                x=self.func(x)
                affinity = self.fc_aff(x)  #affinity values
                pose = self.fc_pose(x) #logit predictions for the pose
                
                return affinity, pose
#################### tensor model-- type H ############################################################################   
            
class Net_H(nn.Module):
        def __init__(self, dims, ch_size):
                super(Net_H, self).__init__()
                self.modules = []
                #nchannels = dims[0]
                fac = int(ch_size/16)
                

                self.func = F.relu

                avgpool1 = nn.AvgPool3d(2,stride=2)
                self.add_module('avgpool_0', avgpool1)
                self.modules.append(avgpool1)
                
                conv1 = SE3Convolution([(28, 0)], [(1*fac, 0), (0*fac, 1), (3*fac , 2)], size=3,stride =1, padding =1)
                self.add_module('unit1_conv',conv1)
                self.modules.append(conv1)
                
                conv2 = SE3Convolution([(1*fac, 0), (0*fac, 1), (3*fac , 2)], [(1*fac, 0), (0*fac, 1), (3*fac,2)], size=1, stride =1, padding =0)
                self.add_module('unit2_conv',conv2)
                self.modules.append(conv2)
                
                avgpool2 = nn.AvgPool3d(2,stride=2)
                self.add_module('avgpool_1', avgpool2)
                self.modules.append(avgpool2)
                
                conv3 = SE3Convolution([(1*fac,0), (0*fac,1), (3*fac,2)], [(2*fac,0), (0*fac,1), (6*fac,2)], size=3, stride =1, padding =1)
                self.add_module('unit3_conv',conv3)
                self.modules.append(conv3)
                
                conv4 = SE3Convolution([(2*fac,0), (0*fac,1), (6*fac,2)], [(2*fac,0), (0*fac,1), (6*fac,2)], size=1  ,stride =1, padding =0)
                self.add_module('unit4_conv',conv4)
                self.modules.append(conv4)
                
                avgpool3 = nn.AvgPool3d(2,stride=2)
                self.add_module('avgpool_2', avgpool3)
                self.modules.append(avgpool3)
                
                conv5 = SE3Convolution([(2*fac,0), (0*fac,1),(6*fac,2)], [(4*fac,0), (0*fac,1), (12*fac,2)], size=3,stride =1, padding =1)
                self.add_module('unit5_conv',conv5)
                self.modules.append(conv5)
                
                div = 2*2*2
                last_size = int(dims[1]//div * dims[2]//div * dims[3]//div * (4*fac*1 + 0*fac*3 + 12*fac*5)      ) 
                
                flattener = View((-1,last_size))
                self.add_module('flatten',flattener)
                self.modules.append(flattener)
                
                self.fc_aff = nn.Linear(last_size,1)
                self.fc_pose = nn.Linear(last_size,1)
                self.add_module('last_fc',self.fc_aff)
                
        def forward(self, x): #should approximate the affinity of the receptor/ligand pair
                for layer in self.modules:
                        x= layer(x)
                        if isinstance(layer,se3cnn.image.convolution.SE3Convolution):
                                x=self.func(x)
                affinity = self.fc_aff(x)  #affinity values
                pose = self.fc_pose(x) #logit predictions for the pose
                
                return affinity, pose