
import torch
import torch.nn as nn
import torch.nn.functional as F
from functools import partial

'''
model is an instance of the class: network
'''
#n_input: atom types
#n_output: label sets
#--^ these might be specific for the dataset they are using
#args: all the user inputs


n_input = validation_set.n_atom_types
n_output = len(validation_set.label_set)


model = network(n_input=n_input, n_output=n_output, args=args)

optimizer = optimizers_L1L2.Adam(param_groups, lr=args.initial_lr)



'''
custom network from Drew:

'''


class CustomResNet(ResNet):    
    def __init__(self,    
                 n_input,    
                 n_output,    
                 args,    
                 module_filters=(8,8,8,8),    
                 filter_factor=2,    
                 final_size=None,    
                 size_resblocks = [3,4,6,2]    
                 ):    
    
        if final_size is None: # Automatically set the final size to the penultimate conv layers sizes * their multiplicity    
            final_size = int(sum([math.ceil((filter_factor**3) * mf*(2*idx+1)) for idx, mf in enumerate(module_filters)]))    
        assert type(module_filters) == tuple, 'needs to be a tuple of length at least 1'    
        assert type(final_size) == int, 'final output needs to be an integer'    
        assert len(size_resblocks) == 4, 'need the size for 4 resblocks'    
            
    
        features = [[[module_filters]],    
                    [[module_filters] * 2] * size_resblocks[0],    
                    [[tuple([math.ceil(filter_factor * mf) for mf in module_filters])] * 2] * size_resblocks[1],    
                    [[tuple([math.ceil((filter_factor ** 2) * mf) for mf in module_filters])] * 2] * size_resblocks[2],    
                    [[tuple([math.ceil((filter_factor ** 3) * mf) for mf in module_filters])] * 2] * size_resblocks[3]     
                            + [[tuple([math.ceil((filter_factor ** 3) * mf) for mf in module_filters]), (final_size,0,0,0)]]]    
        common_params = {    
            'radial_window': partial(gaussian_window_wrapper,    
                                     mode=args.bandlimit_mode, border_dist=0, sigma=0.6),    
            'batch_norm_momentum': 0.01,    
            # TODO: probability needs to be adapted to capsule order    
            'capsule_dropout_p': args.p_drop_conv,  # drop probability of whole capsules    
            'downsample_by_pooling': args.downsample_by_pooling,    
            'normalization': args.normalization    
        }    
        if args.SE3_nonlinearity == 'gated':    
            res_block = SE3GatedResBlock    
        else:    
            res_block = SE3NormResBlock    
        global OuterBlock    
        OuterBlock = partial(OuterBlock,    
                             res_block=partial(res_block, **common_params))
        super().__init__(
            OuterBlock((n_input,),          features[0], size=7),
            OuterBlock(features[0][-1][-1], features[1], size=args.kernel_size, stride=1),
            OuterBlock(features[1][-1][-1], features[2], size=args.kernel_size, stride=2),
            OuterBlock(features[2][-1][-1], features[3], size=args.kernel_size, stride=2),
            OuterBlock(features[3][-1][-1], features[4], size=args.kernel_size, stride=2),
            AvgSpacial(),
            nn.Dropout(p=args.p_drop_fully, inplace=True) if args.p_drop_fully is not None else None,
            nn.Linear(features[4][-1][-1][0], n_output))









'''
network inherit ResNet

'''
class network(ResNet):
    def __init__(self,
                 n_input,
                 n_output,
                 args):

        features = [[[32]],
                    [[32] * 2] * 3,
                    [[64] * 2] * 4,
                    [[128] * 2] * 6,
                    [[256] * 2] * 3]

        common_params = {
            'downsample_by_pooling': args.downsample_by_pooling,
            'conv_dropout_p': args.p_drop_conv,
        }
        global OuterBlock
        OuterBlock = partial(OuterBlock,
                             res_block=partial(ResBlock, **common_params))
        ##^ This set the OuterBlock to be with fixed argument: res_block, so when we use OuterBlock, we only need the other arguments except res_block
        
        #res_block: this is the same with ResBlock, but with parameters set for common_params
        
        
        
        
        ### Below is the blocks in the base class ResNet
        
        
        super().__init__(
            OuterBlock(n_input,             features[0], size=7),
            OuterBlock(features[0][-1][-1], features[1], size=args.kernel_size, stride=1),
            OuterBlock(features[1][-1][-1], features[2], size=args.kernel_size, stride=2),
            OuterBlock(features[2][-1][-1], features[3], size=args.kernel_size, stride=2),
            OuterBlock(features[3][-1][-1], features[4], size=args.kernel_size, stride=2),
            AvgSpacial(),
            nn.Dropout(p=args.p_drop_fully, inplace=True) if args.p_drop_fully is not None else None,
            nn.Linear(features[4][-1][-1], n_output))
        
        
'''
ResNet inherit nn.Module

'''
class ResNet(nn.Module):
    def __init__(self, *blocks): # * multiple arguments (the blocks for ResNet defined by the user)
        super().__init__()
        self.blocks = nn.Sequential(*[block for block in blocks if block is not None])

    def forward(self, x):
        return self.blocks(x)


'''
OuterBlock is used in Network, and it serves as the block in ResNet's arguments
''''
    
class OuterBlock(nn.Module):
    def __init__(self, in_repr, out_reprs, res_block, size=3, stride=1, **kwargs):
        super().__init__()

        reprs = [[in_repr]] + out_reprs

        self.layers = []
        for i in range(len(reprs) - 1):
            self.layers.append(
                res_block(reprs[i][-1], reprs[i+1],
                          size=size,
                          stride=stride if i == 0 else 1,
                          **kwargs)
            )
        self.layers = nn.Sequential(*self.layers)

    def forward(self, x):
        out = self.layers(x)
        return out    


    
    
    
    
class ResBlock(nn.Module):
    def __init__(self, channels_in, channels_out, size=3, stride=1,
                 downsample_by_pooling=False,
                 conv_dropout_p=None):
        super().__init__()

        channels = [channels_in] + channels_out

        self.layers = []
        conv_stride = 1 if downsample_by_pooling else stride
        for i in range(len(channels) - 1):
            self.layers += [
                nn.BatchNorm3d(channels[i]),
                nn.Conv3d(channels[i], channels[i + 1],
                          kernel_size=size,
                          padding=size // 2,
                          stride=conv_stride if i == 0 else 1,
                          bias=False),
                # nn.BatchNorm3d(channels[i + 1])
            ]
            if conv_dropout_p is not None:
                self.layers.append(nn.Dropout3d(p=conv_dropout_p, inplace=True))
            if downsample_by_pooling and i == 0 and stride > 1:
                self.layers.append(nn.AvgPool3d(kernel_size=size,
                                                padding=size//2,
                                                stride=stride))
            if (i + 1) < len(channels) - 1:
                self.layers += [nn.ReLU(inplace=True)]
        self.layers = nn.Sequential(*self.layers)

        self.shortcut = None
        if len(channels_out) > 1:
            if channels_in == channels_out[-1] and stride == 1:
                self.shortcut = lambda x: x
            else:
                self.shortcut = [
                    nn.BatchNorm3d(channels[0]),
                    nn.Conv3d(channels[0], channels[-1],
                              kernel_size=1,
                              padding=0,
                              stride=conv_stride,
                              bias=False),
                    # nn.BatchNorm3d(channels[-1])
                    ]
                if conv_dropout_p is not None:
                    self.shortcut.append(
                        nn.Dropout3d(p=conv_dropout_p, inplace=True))
                if downsample_by_pooling and stride > 1:
                    self.shortcut.append(nn.AvgPool3d(kernel_size=size,
                                                      padding=size // 2,
                                                      stride=stride))
                self.shortcut = nn.Sequential(*self.shortcut)

        self.activation = nn.ReLU(inplace=True)

        # initialize
        for module in self.modules():
            if isinstance(module, nn.Conv3d):
                nn.init.xavier_normal(module.weight.data)
            elif isinstance(module, nn.BatchNorm3d):
                module.weight.data.fill_(1)
                module.bias.data.zero_()

    def forward(self, x):
        out = self.layers(x)
        if self.shortcut is not None:
            out += self.shortcut(x)
        out = self.activation(out)
        return out


