

#import libraries
print("importing packages")

import re
import molgrid
import numpy as np
import torch
import torch.nn as nn
import torch.nn.functional as F
import torch.optim as optim
from torch.nn import init
from torch import autograd
from scipy.stats import pearsonr
import matplotlib.pyplot as plt
import matplotlib as mpl
import seaborn as sns
import matplotlib as mpl
import wandb


config_defaults = {
        'init_channel_size': 16,
        'models' : "scalar",
        'epochs': 1,
        'lr': 0.001,
        'dropout': 0,
        'momentum': 0.9,
        'solver': "adam",
        'batch_norm':0,
        'weight_decay':0,
        'binary_rep':False,
        'batch_size':32   
    }
wandb.login()
wandb.init(config = config_defaults)
config = wandb.config



#Define the path
print("Defining the path")
#0 fold
path_train_lig_cache0 = '/net/pulsar/home/koes/liyeny/dl_project/molcache_file/lig_train0.molcache2'
path_train_rec_cache0 ='/net/pulsar/home/koes/liyeny/dl_project/molcache_file/rec_train0.molcache2'
path_train_type0 = '/net/pulsar/home/koes/liyeny/dl_project/type_files/gaffwmintrain0.types'

path_test_lig_cache0 = '/net/pulsar/home/koes/liyeny/dl_project/molcache_file/lig_test0.molcache2'
path_test_rec_cache0 = '/net/pulsar/home/koes/liyeny/dl_project/molcache_file/rec_test0.molcache2'
path_test_type0 = '/net/pulsar/home/koes/liyeny/dl_project/type_files/gaffwmintest0.types'

#1 fold
path_train_lig_cache1 = '/net/pulsar/home/koes/liyeny/dl_project/molcache_file/lig_train1.molcache2'
path_train_rec_cache1 ='/net/pulsar/home/koes/liyeny/dl_project/molcache_file/rec_train1.molcache2'
path_train_type1 = '/net/pulsar/home/koes/liyeny/dl_project/type_files/gaffwmintrain1.types'

path_test_lig_cache1 = '/net/pulsar/home/koes/liyeny/dl_project/molcache_file/lig_test1.molcache2'
path_test_rec_cache1 = '/net/pulsar/home/koes/liyeny/dl_project/molcache_file/rec_test1.molcache2'
path_test_type1 = '/net/pulsar/home/koes/liyeny/dl_project/type_files/gaffwmintest1.types'

#2 fold
path_train_lig_cache2 = '/net/pulsar/home/koes/liyeny/dl_project/molcache_file/lig_train2.molcache2'
path_train_rec_cache2 ='/net/pulsar/home/koes/liyeny/dl_project/molcache_file/rec_train2.molcache2'
path_train_type2 = '/net/pulsar/home/koes/liyeny/dl_project/type_files/gaffwmintrain2.types'

path_test_lig_cache2 = '/net/pulsar/home/koes/liyeny/dl_project/molcache_file/lig_test2.molcache2'
path_test_rec_cache2 = '/net/pulsar/home/koes/liyeny/dl_project/molcache_file/rec_test2.molcache2'
path_test_type2 = '/net/pulsar/home/koes/liyeny/dl_project/type_files/gaffwmintest2.types'



#Organize data into examples   

def CreateEx(ligmolcache_path, recmolcache_path, type_path):
    
    example = molgrid.ExampleProvider(ligmolcache=ligmolcache_path, recmolcache=recmolcache_path, shuffle=True, default_batch_size=config.batch_size, iteration_scheme = molgrid.IterationScheme.LargeEpoch, duplicate_first=False, balanced=True)
    example.populate(type_path)
    
    return example


train_examples = [CreateEx(path_train_lig_cache0, path_train_rec_cache0, path_train_type0),
                  CreateEx(path_train_lig_cache1, path_train_rec_cache1, path_train_type1),
                  CreateEx(path_train_lig_cache2, path_train_rec_cache2, path_train_type2)]

test_examples = [CreateEx(path_test_lig_cache0, path_test_rec_cache0, path_test_type0), 
                 CreateEx(path_test_lig_cache0, path_test_rec_cache0, path_test_type0),
                 CreateEx(path_test_lig_cache0, path_test_rec_cache0, path_test_type0)]


gmaker = molgrid.GridMaker(binary=config.binary_rep) #binary ture-- point
dims = gmaker.grid_dimensions(14*2) # 14 atoms -- 1 protein 1 ligand # only one rec+onelig per example
tensor_shape = (config.batch_size,)+dims






#define, create, and initialize the model
print("defining, creating, and initializing the model ")  
    

def weights_init(m):
    if isinstance(m, nn.Conv3d) or isinstance(m, nn.Linear):
        init.xavier_uniform_(m.weight.data)
        if m.bias is not None:
            init.constant_(m.bias.data, 0)

if config.models == "scalar":
    from equi_scalar_model import Net
    
elif config.models == "vector":
    from equi_vector_model import Net
     
elif config.models == "tensor":
    from equi_tensor_model import Net
    
elif config.models == "default":
    from default2018_single_model import Net
    
    
    
model = Net(dims, config.init_channel_size)



if torch.cuda.device_count() > 1:
        print("Using {} GPUs".format(torch.cuda.device_count()))
        model = nn.DataParallel(model)
else:
        print('GPUS: {}'.format(torch.cuda.device_count()), flush=True)
        
model.to('cuda:0')
model.apply(weights_init)




#define custom loss
print("defining custom loss")

def new_loss(outputs, labels):
    
    outputs = outputs.flatten()
    labels = labels.flatten()
    
    loss = torch.tensor((0.0), device='cuda')
    count = 0
    
    for output, label in zip(outputs, labels):
        if label < 0:
            abs_label = - label
    
            if output> abs_label:   
                loss += (output-abs_label)**2
                count +=1
        else:
            loss += (output-label)**2  
            count +=1
    return loss/count



#define optimizer and scheduler
print("defining optimizer and scheduler")

optimizer = optim.SGD(model.parameters(), lr=config.lr, weight_decay=config.weight_decay)
if config.solver == "adam":
    optimizer = optim.Adam(model.parameters(), lr=config.lr, weight_decay=config.weight_decay)
    
scheduler = optim.lr_scheduler.StepLR(optimizer, step_size=80, gamma=0.1)



#define zero tensors to put things in
print("defining zero tensors to put things in")

input_tensor = torch.zeros(tensor_shape, dtype=torch.float32, device='cuda')
affinity_labels = torch.zeros(config.batch_size, dtype=torch.float32, device ='cuda')
singleinput = torch.zeros((1,)+dims, dtype=torch.float32, device='cuda')
singleAffinity_label = torch.zeros(1,dtype=torch.float32,device='cuda')




def train_3_fold():
    for fold in range(3):
        traine = train_examples[fold]
        teste = test_examples[fold]
        model,testlabels,results,mses,ames,pearsons,losses = train_single_fold(model, traine, teste)
        ######???
     
    return 
        
        






def train_single_fold(model,traine, teste):
   
    wandb.run.name = config.models + "/" + str(config.init_channel_size)
    
    losses = [] #losses for training for each batch 
    mses = [] #mean square error of test for each epoch during training 
    ames = [] #absolute mean error of test for each epoch during training
    pearsons = [] #correlation of test set(all samples) for each epoch during training

    for e in range(config.epochs): 
        for idx, batch in enumerate(traine):
            optimizer.zero_grad()
            batch.extract_label(1, affinity_labels)
            labels = affinity_labels.float().to('cuda')
            gmaker.forward(batch, input_tensor, random_translation=2, random_rotation=True)
            output = model(input_tensor)
            loss = new_loss(output,labels)
            loss.backward()
            nn.utils.clip_grad_norm_(model.parameters(),1)
            optimizer.step()
            losses.append(float(loss))
            wandb.log({"train/loss":loss})

        scheduler.step()
        #eval performance on test set
        with torch.no_grad():
            results = []
            testlabels = []
            for idx, batch in enumerate(teste):
                batch.extract_label(1,singleAffinity_label)
                if singleAffinity_label < 0:
                    continue  
                gmaker.forward(batch,singleinput)
                output = model(singleinput) 
                results.append(output.detach().cpu().numpy())
                testlabels.append(singleAffinity_label.detach().cpu().numpy()) 

            results = np.array(results).flatten()
            testlabels = np.array(testlabels).flatten()
                     
            valrmse = np.sqrt(np.mean((results - testlabels)**2))  
            if np.isinf(valrmse):
                valrmse = 1000
            valame = np.mean(np.abs(results-testlabels))
            R = pearsonr(results,testlabels)[0]
            mses.append(valrmse)
            ames.append(valame)
            pearsons.append(R)
            wandb.log({"val/mses":valrmse, "epoch": e})
            wandb.log({"val/ames":valame, "epoch": e})
            wandb.log({"val/pearsons":R, "epoch": e})
        
           

    return model,testlabels,results,mses,ames,pearsons,losses




        



train_3_fold()







